#!/usr/bin/env python2

import matplotlib
matplotlib.use('Agg')

import os
import sys
import pysam
import pybedtools
import metaseq
import gzip
import numpy as np
import pandas as pd
import scipy.stats
import argparse
import logging
import re
from multiprocessing import cpu_count
from matplotlib import pyplot as plt
from matplotlib import mlab


EPILOG = '''
For more details:
        %(prog)s --help
'''


def get_args():
    '''Define arguments.'''

    parser = argparse.ArgumentParser(
        description=__doc__, epilog=EPILOG,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('-b', '--bam',
                        help="The bam file to qc on.",
                        required=True)

    parser.add_argument('-p', '--prefix',
                        help="The Prefix of the file.",
                        required=True)

    parser.add_argument('-t', '--tss',
                        help="The TSS file to use as QC.",
                        required=True)

    parser.add_argument('-c', '--chromsizes',
                        help="The chromsizes file to use as QC.",
                        required=True)

    args = parser.parse_args()
    return args


def main():
    args = get_args()
    prefix = args.prefix
    bam = args.bam
    tss = args.tss
    chromsizes = args.chromsizes

    bins=400
    bp_edge = 2000

    tss = pybedtools.BedTool(tss)
    tss_ext = tss.slop(b=bp_edge, g=chromsizes)

    bam = metaseq.genomic_signal(bam, 'bam') # Need to shift reads and just get ends, just load bed file?
    bam_array = bam.array(tss_ext, bins=bins, shift_width = -75/2, stranded=True, processes=cpu_count())

    tss_plot_file = '{0}_tss-enrich.png'.format(prefix)
    tss_plot_large_file = '{0}_large_tss-enrich.png'.format(prefix)

    bp_edge=2000
    num_edge_bins = int(100/(2*bp_edge/bins))
    bin_means = bam_array.mean(axis=0)
    avg_noise = (sum(bin_means[:num_edge_bins]) +
                 sum(bin_means[-num_edge_bins:]))/(2*num_edge_bins)
    bam_array /= avg_noise

    # Generate a line plot
    fig = plt.figure()
    ax = fig.add_subplot(111)
    x = np.linspace(-bp_edge, bp_edge, bins)

    ax.plot(x, bam_array.mean(axis=0), color='r', label='Mean')
    ax.axvline(0, linestyle=':', color='k')

    # Note the middle high point (TSS)
    tss_point_val = max(bam_array.mean(axis=0))

    ax.set_xlabel('Distance from TSS (bp)')
    ax.set_ylabel('Average read coverage (per million mapped reads)')
    ax.legend(loc='best')

    fig.savefig(tss_plot_file)

    # Print a more complicated plot with lots of info

    # Find a safe upper percentile - we can't use X if the Xth percentile is 0
    upper_prct = 99
    if mlab.prctile(bam_array.ravel(), upper_prct) == 0.0:
        upper_prct = 100.0

    plt.rcParams['font.size'] = 8
    fig = metaseq.plotutils.imshow(bam_array,
                                   x=x,
                                   figsize=(5, 10),
                                   vmin=5, vmax=upper_prct, percentile=True,
                                   line_kwargs=dict(color='k', label='All'),
                                   fill_kwargs=dict(color='k', alpha=0.3),
                                   sort_by=bam_array.mean(axis=1))

    # And save the file
    fig.savefig(tss_plot_large_file)

    print(tss_point_val)


if __name__ == '__main__':
    main()
